from enum import Enum, EnumMeta
from typing import Any

class EnumeratorMeta(EnumMeta):
    def __contains__(cls, member: Any):
        if type(member) == cls:
            return EnumMeta.__contains__(cls, member)
        else:
            try:
                cls(member)
            except ValueError:
                return False
            return True

    def __getattribute__(cls, name):
        value = super().__getattribute__(name)
        if isinstance(value, cls):
            value = value.value
        return value
        
class Enumerator(Enum, metaclass=EnumeratorMeta):
    pass

class SFWGif(Enumerator):
    """
    SFW ANIME GIFs Types
    """
    ANGRY: str = "angry"
    BAKA: str = "baka"
    BITE: str = "bite"
    BLUSH: str = "blush"
    CRY: str = "cry"
    DANCE: str = "dance"
    DEREDERE: str = "deredere"
    HAPPY: str = "happy"
    HUG: str = "hug"
    KISS: str = "kiss"
    PAT: str = "pat"
    PUNCH: str = "punch"
    SLAP: str = "slap"
    SLEEP: str = "sleep"
    SMUG: str = "smug"


class NSFWGif(Enumerator):
    """
    NSFW ANIME GIFs Types
    """
    ANAL: str = "anal"
    FUCK: str = "fuck"
    SUCK: str = "suck"

class SearchType(Enumerator):
    """
    Manga or Anime
    """
    ANIME: str = "anime"
    MANGA: str = "manga"



import re, typing, aiohttp

class SearchError(Exception):
    pass
class GifError(Exception):
    pass
    
async def Request(url: str, params: dict = {}, headers: dict = {}, like: str = 'json'):
    try:
        async with aiohttp.ClientSession(headers=headers) as session:
            async with session.get(url=url, params=params) as r:
                if r.status != 200 and r.status != 201:
                    return None
                if like.lower() == 'json':
                    return await r.json()
                elif like.lower() == 'text':
                    return await r.text()
                elif like.lower() == 'read':
                    return await r.read()
                else:
                    raise SyntaxError('Invalid like method provided in request')
    except Exception as e:
        raise e
        return None

def camel_to_snake(s) -> str:
    return re.sub("([A-Z]\w+$)", "_\\1", s).lower()

def to_snake(d) -> typing.Union[dict, list]:
    """
    Converts a dict or list to snake case from camel case
    """
    if isinstance(d, list):
        return [to_snake(i) if isinstance(i, (dict, list)) else i for i in d]
    return {
        camel_to_snake(a): to_snake(b) if isinstance(b, (dict, list)) else b
        for a, b in d.items()
    }




from dataclasses import dataclass

@dataclass
class Titles:
    native: str
    romaji: str
    english: str

@dataclass
class GifResult:
    """
    Object representation of an anime gif result
    """
    url: str
    title: Titles
    episode: int

@dataclass
class ImageCover:
    tiny: str
    small: str
    medium: str
    large: str
    original: str

@dataclass
class SearchAttributes:
    created_at: str
    updated_at: str
    synopsis: str
    description: str
    titles: Titles
    canonical_title: str
    avarage_rating: str
    user_count: int
    favorites_count: int
    start_date: str
    end_date: str
    next_relase: str
    popularity_ranking: str
    rating_rank: int
    age_rating: str
    cover_image: ImageCover
    poster_image: ImageCover
    
@dataclass
class SearchResult:
    """
    Object representation of an anime search rsult 
    """
    id: int
    type: str
    links: dict
    attributes: SearchAttributes